﻿using UnityEngine;
using System.Collections;
using System.Collections.Generic;

public class BoardController : MonoBehaviour {
	public AudioClip blueMove;
	public AudioClip redMove;
	public AudioClip getPoint;
	public AudioClip flip;
	public AudioClip tumble;

	public const int PEG_DIM = 6;
	public bool[,] grid;
	public GameObject[,] pegs;
	public GameObject pegProto;
	public int redX; 
	public int redY; 
	public int blueX; 
	public int blueY;
	public GameObject redsideDot;
	public GameObject bluesideDot;
	public GameObject targetMarker;
	public bool boardIsRedside = true;
	public int targetX;
	public int targetY;

	public int movesRemaining;
	public int MAX_MOVES = 100;
	public bool gameOver;

	public int score;

	 float springPos;
	 float springVel;
	 float springDamp = 5f;
	 float springTension = 40;

	float swivelSpringPos;
	float swivelSpringVel;
	float swivelSpringDamp = 13f;
	float swivelSpringTension = 150;

	private float swivel = 0;

	//public Material activePeg;
	//public Material inactivePeg;

	public GUIStyle instructionsStyle;
	public GUIStyle scoreStyle;
	public GUIStyle titleStyle;
	public GUIStyle gameOverStyle;
	public GUIStyle creditStyle;

	public Texture justWhite;

	public static float g2b(int coord)
	{
		return coord*1.0f/PEG_DIM+1.0f/(PEG_DIM*2)-0.5f;
	}

	void OnGUI()
	{
		if(Event.current.type.Equals(EventType.Repaint))
		{
			Rect upperLeft = new Rect(4,4,0,0);
			GUI.Label(upperLeft, "Flopswitch", titleStyle);
			Rect upperLeftCredit = new Rect(4, 60, 0, 0);
			GUI.Label(upperLeftCredit, "created by Nathan McCoy\nfor Ludum Dare 30", creditStyle);

			Rect lowerLeft = new Rect(4, Screen.height-4, 0, 0);
			GUI.Label(lowerLeft, "Q, W, A, S to move\nSpace to flip", instructionsStyle);

			Rect upperRight = new Rect(Screen.width-4, 4, 0, 0);
			if(movesRemaining <= 20) GUI.color = Color.yellow;
			GUI.Label(upperRight, "Moves Remaining: "+movesRemaining, scoreStyle);
			GUI.color = Color.white;
			GUI.Label(upperRight, "\nScore: "+score, scoreStyle);

			if(gameOver)
			{
				Rect fullScreen = new Rect(0, 0, Screen.width, Screen.height);
				GUI.color = new Color(0, 0, 0, 0.5f);
				GUI.DrawTexture(fullScreen, justWhite);
				GUI.color = Color.white;
				Rect center = new Rect(Screen.width/2, Screen.height/2, 0,0);
				if(movesRemaining <= 0) GUI.Label(center, "GAME OVER\nFinal Score: "+score+"\n\npress R to retry", gameOverStyle);
				else GUI.Label(center, "GAME OVER\n(no legal moves!)\n\npress R to retry", gameOverStyle);
			}
		}
	}

	public void setRandomNewTarget(bool side)
	{
		List<Vector2> candidates = new List<Vector2>();
		for(int x=0; x<PEG_DIM; x++)for(int y=0; y<PEG_DIM; y++)
		{
			if((grid[x,y] == side) && !(redX == x && redY == y) && !(blueX == x && blueY == y)) candidates.Add(new Vector2(x, y));
		}
		if(candidates.Count == 0) setRandomNewTarget(!side);
		else
		{
			int which = (int)(Random.value*candidates.Count);
			if(which == candidates.Count) which--; //Unity randomness is dumb
			Vector2 target = candidates[which];
			targetX = (int)target.x;
			targetY = (int)target.y;
		}
		targetMarker.transform.localPosition = new Vector3(g2b(targetX), 0, g2b(targetY));
	}

	// Use this for initialization
	void Start () {

		springPos = 20;
		springVel = 0;
		//swivelSpringPos = 90;
		score = 0;
		boardIsRedside = true;
		grid = new bool[PEG_DIM,PEG_DIM];
		if(pegs == null)
		{
			pegs = new GameObject[PEG_DIM,PEG_DIM];
			for(int y = 0; y < PEG_DIM; y++)for(int x = 0; x < PEG_DIM; x++)
			{
				if(x < PEG_DIM/2) grid[x,y] = true;
				pegs[x,y] = Instantiate(pegProto) as GameObject;
				pegs[x,y].transform.parent = transform;
				ProtoToggle prt = pegs[x,y].GetComponent<ProtoToggle>();
				prt.x = x;
				prt.y = y;
			}
		}
		else
		{
			for(int y = 0; y < PEG_DIM; y++)for(int x = 0; x < PEG_DIM; x++)
			{
				if(x < PEG_DIM/2) grid[x,y] = true;
				else grid[x,y] = false;
			}
		}

		redY = PEG_DIM-1;
		redX = PEG_DIM-1;
		blueX = 0;
		blueY = 0;
		adjustDots();
		setRandomNewTarget(false);

		movesRemaining = MAX_MOVES;
		gameOver = false;

		audio.PlayOneShot(tumble, 1);
	}

	public void adjustDots()
	{
		redsideDot.transform.localPosition = new Vector3(g2b(redX), 1.25f, g2b(redY));
		bluesideDot.transform.localPosition = new Vector3(g2b(blueX), -1.25f, g2b(blueY));
	}

	/*
	public void adjustPeg(int x, int y)
	{
		float fx = g2b(x);
		float fy = g2b(y);
		float fd = -0.5f;
		if(grid[x,y]) fd = 0.5f;
		pegs[x,y].transform.localPosition = new Vector3(fx, fd, fy);
		if((x==redX && y==redY)||(x==blueX && y==blueY)) pegs[x,y].renderer.material = activePeg;
		else pegs[x,y].renderer.material = inactivePeg;
	}*/

	public bool anyLegalMoves()
	{
		if(isMoveLegal(true, 1, 0)) return true;
		if(isMoveLegal(true, -1, 0)) return true;
		if(isMoveLegal(true, 0, -1)) return true;
		if(isMoveLegal(true, 0, 1)) return true;
		if(isMoveLegal(false, 1, 0)) return true;
		if(isMoveLegal(false, -1, 0)) return true;
		if(isMoveLegal(false, 0, -1)) return true;
		if(isMoveLegal(false, 0, 1)) return true;
		return false;
	}

	public bool isMoveLegal(bool side, int x, int y)
	{
		if(side)
		{
			int newX = x + redX;
			int newY = y + redY;
			if(newX < 0 || newX >= PEG_DIM || newY < 0 || newY >= PEG_DIM) return false;
			if(grid[newX, newY]) return false;
		}
		else
		{
			int newX = x + blueX;
			int newY = y + blueY;
			if(newX < 0 || newX >= PEG_DIM || newY < 0 || newY >= PEG_DIM) return false;
			if(!grid[newX, newY]) return false;
		}
		return true;
	}

	public bool tryDotMove(int x, int y)
	{
		if(boardIsRedside)
		{
			int newX = x + redX;
			int newY = y + redY;
			if(newX < 0 || newX >= PEG_DIM || newY < 0 || newY >= PEG_DIM) return false;
			if(grid[newX, newY]) return false;
			// OK
			grid[redX, redY] = true;

			redX = newX;
			redY = newY;
			adjustDots();
			//adjustPeg(redX-x, redY-y);
			//adjustPeg(redX, redY);
			return true;
		}
		else
		{
			int newX = x + blueX;
			int newY = y + blueY;
			if(newX < 0 || newX >= PEG_DIM || newY < 0 || newY >= PEG_DIM) return false;
			if(!grid[newX, newY]) return false;
			// OK
			grid[blueX, blueY] = false;
			blueX = newX;
			blueY = newY;
			adjustDots();
			//adjustPeg(blueX-x, blueY-y);
			//adjustPeg(blueX, blueY);
			return true;
		}

	}
	
	// Update is called once per frame
	void Update () {

		//do spawn effects
		if(boardIsRedside) swivel = 0;
		else swivel = 180;

		springVel += (0 - springPos)*Time.deltaTime*springTension;
		springVel *= (1-springDamp*Time.deltaTime);
		springPos += springVel*Time.deltaTime;

		swivelSpringVel += (swivel - swivelSpringPos)*Time.deltaTime*swivelSpringTension;
		swivelSpringVel *= (1-swivelSpringDamp*Time.deltaTime);
		swivelSpringPos += swivelSpringVel*Time.deltaTime;

		transform.position = new Vector3(0, springPos, 0);

		int boardInvert = 1;
		if(!boardIsRedside) boardInvert = -1;
		bool moved = false;
		if(!gameOver)
		{
			if(Input.GetKeyDown(KeyCode.Q)) moved = moved || tryDotMove(-1*boardInvert, 0);
			if(Input.GetKeyDown(KeyCode.W)) moved = moved || tryDotMove(0, 1);
			if(Input.GetKeyDown(KeyCode.A)) moved = moved || tryDotMove(0, -1);
			if(Input.GetKeyDown(KeyCode.S)) moved = moved || tryDotMove(1*boardInvert, 0);

			if(moved)
			{
				if(boardIsRedside) audio.PlayOneShot(redMove, 0.4f);
				else audio.PlayOneShot(blueMove, 0.4f);

			}

			if(Input.GetKeyDown(KeyCode.Space)) 
			{
				boardIsRedside = !boardIsRedside;
				audio.PlayOneShot(flip, 1);

			}
		}
		if(moved) movesRemaining--;

		transform.rotation = Quaternion.Euler(new Vector3(0, 0, springPos*20+swivelSpringPos));

		if(moved && ((redX == targetX && redY == targetY)||(blueX == targetX && blueY == targetY))) 
		{
			score++;
			setRandomNewTarget(boardIsRedside);
			audio.PlayOneShot(getPoint, 0.7f);
		}

		if(movesRemaining <=0) gameOver = true;
		if(!anyLegalMoves()) gameOver = true;
		if(moved && !anyLegalMoves()) Debug.Log("Game over!");

		if(Input.GetKeyDown(KeyCode.R)) Start();

	}
}
